\name{simpcond}
\alias{exception}
\alias{simple.exception}
\alias{simple.warning}
\alias{signal.condition}
\alias{with.handlers}
\alias{abort}
\alias{muffle.warnings}
\alias{ignore.errors}

\title{Simple Condition Handling Mechanism}
\description{
  A simple condition handling mechanism for R.  This is in no sense
  meant as a final implementation but rather as a test bed for trying out
  ideas for how this might work.
}

\usage{
    simple.warning(message, call = NULL)
    simple.exception(message, call = NULL)
    signal.condition(c)
    try.catch(expr, ..., finally = NULL)
    with.handlers(expr, ...)
    abort(restarts.honored = FALSE)
    muffle.warnings(expr)
    ignore.errors(expr)
}

\arguments{
  \item{string}{ string describing the exception }
  \item{call}{ a call expression }
  \item{cond}{condition object}
  \item{expr}{expression to execute for value}
  \item{finally}{expression to execute before returning or continuing a throw}
  \item{...}{named arguments with names representing condition classes and
             values that are functions of one argument} 
  \item{restarts.honored}{boolean indicating whether jump should stop at
                          restarted function frame}
}

\details{

   Conditions are objects inheriting from the abstract class
   \code{contition}.  Exceptions and warnings are objects inheriting
   from the abstract subclasses \code{exception} and \code{warning}.
   The class \code{simple.exception} is the class currently used by
   \code{stop} and all internal error signals.  Similarly,
   \code{simple.warning} is used by \code{warning}.  The constructors
   by the same names take a string describing the exception as
   argument and an optional call returns a \code{simple.exception}
   object.

   Conditions are signaled by \code{signal.condition}.  In addition,
   the \code{stop} and \code{warning} functions have been modified to
   also accept condition arguments.  

   The function \code{try.catch} evaluates its expression argument
   in a context where the handlers provided in the \code{...} argument
   are available.  The \code{finally} expression is then evaluated in
   the context in which \code{try.catch} was called; that is, the
   handlers supplied to the current \code{try.catch} call are not
   active when the \code{finally} expression is evaluated.

   Handlers provided in the \code{...} argument to \code{try.catch}
   are established for the duration of the evaluation of \code{expr}.
   If no condition is signaled when evaluating \code{expr} then
   \code{try.catch} returns the value of the expression.

   If a condition is signaled while evaluating \code{expr} then
   established handlers are checked, starting with the most recently
   established ones, for one matching the class of the condition.  If
   a handler is found then control is transferred to the
   \code{try.catch} call that established the handler, the handlers in
   that call are disestablished, the handler is called with the
   condition as its argument, and the result returned by the handler is
   returned as the value of the \code{try.catch} call.

   Default handlers are provided for warnings and exceptions.
   For all other contidions, an exception is signaled if no handler is
   found.

   Calling handlers are established by \code{with.handlers}.  If
   a condition is signaled and the applicable handler is a calling
   handler, then the handler is called by \code{signal.condition} in
   the context where the condition was signaled but with the available
   handlers restricted to those below the handler called in the
   handler stack.  If the handler returns, then its return value is
   returned by \code{signal.condition}.

   \code{abort} is used by \code{stop} to jump to the top level (or to
   a function frame where \code{restart} has been called, if there is
   one.  For proper interaction with \code{restart}, code implementing
   a jump to top level as part of an exception handler should be sure to
   honor any restarts that are on the stack.

   \code{muffle.warnings} evaluates its expression in a context that
   ignores all warnings.  \code{ignore.errors} is analogous to
   \code{try}: it evaluates the expression and, if no exception is
   raised, returns the result.  If an exception is raised then it
   returns the exception object.
}

\references{The \code{try.catch} mechanism is similar to Java
  exception handling.  Calling handlers are based on Common Lisp and
  Dylan}

\author{Luke Tierney}

\examples{
try.catch(1, finally=print("Hello"))
e<-simple.exception("test exception")
\dontrun{stop(e)}
\dontrun{try.catch(stop(e), finally=print("Hello"))}
\dontrun{toplevel(try.catch(stop("fred"), finally=print("Hello")))}
try.catch(stop(e), exception = function(e) e, finally=print("Hello"))
try.catch(stop("fred"),  exception = function(e) e, finally=print("Hello"))
with.handlers({ warning("A"); 1+2 }, warning = function(w) {})
}
